@@ -59,7 +59,7 @@ static int
 i386_linux_register_reggroup_p (struct gdbarch *gdbarch, int regnum,
 				const struct reggroup *group)
 {
-  if (regnum == I386_LINUX_ORIG_EAX_REGNUM)
+  if (regnum == I386_LINUX_ORIG_EAX_REGNUM || i386_is_tls_regnum_p (regnum))
     return (group == system_reggroup
 	    || group == save_reggroup
 	    || group == restore_reggroup);
@@ -899,7 +899,7 @@ i386_linux_intx80_sysenter_syscall_record (struct regcache *regcache)
     {
       gdb_printf (gdb_stderr,
 		  _("Process record and replay target doesn't "
-		    "support syscall number %s\n"), 
+		    "support syscall number %s\n"),
 		  plongest (syscall_native));
       return -1;
     }
@@ -1044,7 +1044,10 @@ int i386_linux_gregset_reg_offset[] =
   -1, -1, -1, -1, -1, -1, -1, -1, /* k0 ... k7 (AVX512)  */
   -1, -1, -1, -1, -1, -1, -1, -1, /* zmm0 ... zmm7 (AVX512)  */
   -1,				  /* PKRU register  */
+  -1,				  /* SSP register.  */
+  -1, -1,			  /* fs/gs base registers.  */
   11 * 4,			  /* "orig_eax"  */
+  -1, -1, -1,			  /* TLS GDT regs: i386_tls_gdt_0...2.  */
 };
 
 /* Mapping between the general-purpose registers in `struct
@@ -1105,11 +1108,10 @@ i386_linux_core_read_xsave_info (bfd *abfd, x86_xsave_layout &layout)
 /* See i386-linux-tdep.h.  */
 
 bool
-i386_linux_core_read_x86_xsave_layout (struct gdbarch *gdbarch,
+i386_linux_core_read_x86_xsave_layout (struct gdbarch *gdbarch, bfd &cbfd,
 				       x86_xsave_layout &layout)
 {
-  return i386_linux_core_read_xsave_info (current_program_space->core_bfd (),
-					  layout) != 0;
+  return i386_linux_core_read_xsave_info (&cbfd, layout) != 0;
 }
 
 /* See arch/x86-linux-tdesc.h.  */
@@ -1162,15 +1164,150 @@ i386_linux_collect_xstateregset (const struct regset *regset,
   i387_collect_xsave (regcache, regnum, xstateregs, 1);
 }
 
+/* Within a tdep file we don't have access to system headers.  This
+   structure is a clone of 'struct user_desc' from 'asm/ldt.h' on x86
+   GNU/Linux systems.  See 'see man 2 get_thread_area' on a suitable x86
+   machine for more details.  */
+
+struct x86_user_desc
+{
+  uint32_t  entry_number;
+  uint32_t  base_addr;
+  uint32_t  limit;
+
+  /* In the actual struct, these flags are a series of 1-bit separate
+     flags.  But we don't need that level of insight for the
+     processing we do in GDB, so just make it a single field.  */
+  uint32_t flags;
+};
+
+/* Supply the 3 tls related registers from BUFFER (length LEN) into
+   REGCACHE.  The REGSET and REGNUM are ignored, all three registers are
+   always supplied from BUFFER.  */
+
+static void
+i386_linux_supply_tls_regset (const regset *regset,
+			      regcache *regcache, int regnum,
+			      const void *buffer, size_t len)
+{
+  gdbarch *gdbarch = regcache->arch ();
+  i386_gdbarch_tdep *tdep = gdbarch_tdep<i386_gdbarch_tdep> (gdbarch);
+
+  if (!tdep->i386_linux_tls)
+    return;
+
+  gdb_assert (len == sizeof (x86_user_desc) * 3);
+
+  for (int i = 0; i < 3; ++i)
+    {
+      int tls_regno = I386_LINUX_TLS_GDT_0 + i;
+
+      gdb_assert (regcache->register_size (tls_regno)
+		  == sizeof (x86_user_desc));
+
+      regcache->raw_supply (tls_regno, buffer);
+      buffer = static_cast<const x86_user_desc *> (buffer) + 1;
+    }
+}
+
+/* Collect the 3 tls related registers from REGCACHE, placing the results
+   in to BUFFER (length LEN).  The REGSET and REGNUM are ignored, all three
+   registers are always collected from REGCACHE.  */
+
+static void
+i386_linux_collect_tls_regset (const regset *regset,
+			       const regcache *regcache,
+			       int regnum, void *buffer, size_t len)
+{
+  gdbarch *gdbarch = regcache->arch ();
+  i386_gdbarch_tdep *tdep = gdbarch_tdep<i386_gdbarch_tdep> (gdbarch);
+
+  if (!tdep->i386_linux_tls)
+    return;
+
+  gdb_assert (len == sizeof (x86_user_desc) * 3);
+
+  for (int i = 0; i < 3; ++i)
+    {
+      x86_user_desc desc;
+      int tls_regno = I386_LINUX_TLS_GDT_0 + i;
+
+      gdb_assert (regcache->register_size (tls_regno) == sizeof (desc));
+
+      regcache->raw_collect (tls_regno, &desc);
+      memcpy (buffer, &desc, sizeof (desc));
+      buffer = static_cast<x86_user_desc *> (buffer) + 1;
+    }
+}
+
 /* Register set definitions.  */
 
-static const struct regset i386_linux_xstateregset =
+static const regset i386_linux_xstateregset =
   {
     NULL,
     i386_linux_supply_xstateregset,
     i386_linux_collect_xstateregset
   };
 
+static const regset i386_linux_tls_regset =
+  {
+    NULL,
+    i386_linux_supply_tls_regset,
+    i386_linux_collect_tls_regset
+  };
+
+/* Helper for i386_linux_iterate_over_regset_sections.  Should we
+   visit the NT_386_TLS note?  If REGCACHE is NULL then we are reading
+   the notes from the corefile, so we always visit the note.  If
+   REGCACHE is not NULL, in this case we are creating a corefile.  In
+   this case, we only visit the note if all the TLS registers are
+   valid, and their base address and limit are not zero, this mirrors
+   the kernel behaviour where the TLS note is elided when the TLS GDT
+   entries have not been set.
+
+   Only call for architectures where i386_gdbarch_tdep::i386_linux_tls
+   is true.  */
+
+static bool
+should_visit_i386_tls_note (const regcache *regcache)
+{
+  if (regcache == nullptr)
+    return true;
+
+  /* Check the pre-condition.  */
+  gdbarch *gdbarch = regcache->arch ();
+  i386_gdbarch_tdep *tdep = gdbarch_tdep<i386_gdbarch_tdep> (gdbarch);
+  gdb_assert (tdep->i386_linux_tls);
+
+  for (int i = 0; i < 3; ++i)
+    {
+      int tls_regno = I386_LINUX_TLS_GDT_0 + i;
+
+      /* If we failed to read any of the registers then we'll not be
+	 able to emit valid note.  */
+      if (regcache->get_register_status (tls_regno) != REG_VALID)
+	return false;
+
+      /* As i386_gdbarch_tdep::i386_linux_tls is true, the registers
+	 must be the right size.  The flag is only set true when this
+	 condition holds.  */
+      gdb_assert (regcache->register_size (tls_regno)
+		  == sizeof (x86_user_desc));
+
+      /* Read the TLS GDT entry.  If it is in use then we want to
+	 write the NT_386_TLS note.  */
+      x86_user_desc ud;
+      regcache->raw_collect (tls_regno, &ud);
+      if (ud.base_addr != 0 && ud.limit != 0)
+	return true;
+    }
+
+  /* Made it through the loop without finding any in-use TLS related
+     GDT entries.  No point creating the NT_386_TLS note, the kernel
+     doesn't.  */
+  return false;
+}
+
 /* Iterate over core file register note sections.  */
 
 static void
@@ -1192,12 +1329,15 @@ i386_linux_iterate_over_regset_sections (struct gdbarch *gdbarch,
 	cb_data);
   else
     cb (".reg2", 108, 108, &i386_fpregset, NULL, cb_data);
+
+  if (tdep->i386_linux_tls && should_visit_i386_tls_note (regcache))
+    cb (".reg-i386-tls", 48, 48, &i386_linux_tls_regset, nullptr, cb_data);
 }
 
 /* Linux kernel shows PC value after the 'int $0x80' instruction even if
    inferior is still inside the syscall.  On next PTRACE_SINGLESTEP it will
    finish the syscall but PC will not change.
-   
+
    Some vDSOs contain 'int $0x80; ret' and during stepping out of the syscall
    i386_displaced_step_fixup would keep PC at the displaced pad location.
    As PC is pointing to the 'ret' instruction before the step
@@ -1205,7 +1345,7 @@ i386_linux_iterate_over_regset_sections (struct gdbarch *gdbarch,
    and PC should not be adjusted.  In reality it finished syscall instead and
    PC should get relocated back to its vDSO address.  Hide the 'ret'
    instruction by 'nop' so that i386_displaced_step_fixup is not confused.
-   
+
    It is not fully correct as the bytes in struct
    displaced_step_copy_insn_closure will not match the inferior code.  But we
    would need some new flag in displaced_step_copy_insn_closure otherwise to
@@ -1271,6 +1411,37 @@ i386_linux_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
   if (!valid_p)
     return;
 
+  /* Helper function.  Look for TLS_REG_NAME in I386_FEATURE (with the
+     associated LOCAL_TDESC_DATA), and if the register is found assign it
+     TLS_REGNO.  Return true if the register is found, and it is the size
+     of 'struct user_desc' (see man 2 get_thread_area), otherwise, return
+     false.  */
+  static const auto valid_tls_reg
+    = [] (const tdesc_feature *i386_feature,
+	  tdesc_arch_data *local_tdesc_data,
+	  const char *tls_reg_name, int tls_regno) -> bool
+  {
+    static constexpr int required_reg_size
+      = sizeof (x86_user_desc) * HOST_CHAR_BIT;
+    return (tdesc_numbered_register (i386_feature, local_tdesc_data,
+				     tls_regno, tls_reg_name)
+	    && (tdesc_register_bitsize (i386_feature, tls_reg_name)
+		== required_reg_size));
+  };
+
+  /* Check all the expected tls related registers are found, and are the
+     correct size.  If they are then mark the tls feature as being active
+     in TDEP.  Otherwise, leave the feature as deactivated.  */
+  valid_p = (valid_tls_reg (feature, tdesc_data, "i386_tls_gdt_0",
+			    I386_LINUX_TLS_GDT_0)
+	     && valid_tls_reg (feature, tdesc_data, "i386_tls_gdt_1",
+			       I386_LINUX_TLS_GDT_1)
+	     && valid_tls_reg (feature, tdesc_data, "i386_tls_gdt_2",
+			       I386_LINUX_TLS_GDT_2));
+
+  if (valid_p)
+    tdep->i386_linux_tls = true;
+
   /* Add the %orig_eax register used for syscall restarting.  */
   set_gdbarch_write_pc (gdbarch, i386_linux_write_pc);
 
@@ -1492,6 +1663,9 @@ i386_linux_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
 
 INIT_GDB_FILE (i386_linux_tdep)
 {
+  gdb_assert (ARRAY_SIZE (i386_linux_gregset_reg_offset)
+	      == I386_LINUX_NUM_REGS);
+
   gdbarch_register_osabi (bfd_arch_i386, 0, GDB_OSABI_LINUX,
 			  i386_linux_init_abi);
 }
