@@ -1,6 +1,6 @@
 /* Common definitions.
 
-   Copyright (C) 1986-2024 Free Software Foundation, Inc.
+   Copyright (C) 1986-2025 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -27,6 +27,14 @@
 #pragma GCC optimize("-fno-hoist-adjacent-loads")
 #endif
 
+#if defined (__GNUC__) && !defined (__clang__) \
+  && ((__GNUC__ >= 12 && __GNUC__ <= 15)       \
+      || (__GNUC__ == 16 && __GNUC_MINOR__ < 1))
+/* Work around PR gcc/120987 starting gcc 12, and assume it will be fixed in
+   the gcc 16.1 release.  */
+#pragma GCC optimize("-fno-ipa-modref")
+#endif
+
 #include <gdbsupport/config.h>
 
 #undef PACKAGE_NAME
@@ -153,7 +161,7 @@
    and builds with -O2, and ... the assert doesn't trigger, because it's
    optimized away by gcc.
 
-   There's no suppported recipe to prevent the assertion from being optimized
+   There's no supported recipe to prevent the assertion from being optimized
    away (other than: build with -O0, or remove the nonnull attribute).  Note
    that -fno-delete-null-pointer-checks does not help.  A patch was submitted
    to improve gcc documentation to point this out more clearly (
@@ -208,13 +216,18 @@
 #include "errors.h"
 #include "print-utils.h"
 #include "common-debug.h"
-#include "cleanups.h"
 #include "common-exceptions.h"
 #include "gdbsupport/poison.h"
 
 /* Pull in gdb::unique_xmalloc_ptr.  */
 #include "gdbsupport/gdb_unique_ptr.h"
 
+/* Note that there's no simple way to enforce the use of the c-ctype
+   functions.  We can't poison the <ctype.h> functions (see
+   safe-ctype.h) because that will provoke errors from libstdc++
+   headers.  */
+#include "c-ctype.h"
+
 /* sbrk on macOS is not useful for our purposes, since sbrk(0) always
    returns the same value.  brk/sbrk on macOS is just an emulation
    that always returns a pointer to a 4MB section reserved for
